/*
 *  Copyright(c)2002 Forward Computing and Control Pty. Ltd.
 *  ACN 003 669 994, NSW, Australia      All rights Reserved
 *  
 *   Written by Dr. M.P. Ford
 *
 *  ThreadReturn package Copyright (c) 2003 Forward Computing and Control Pty. Ltd.
 *  NSW. Australia,  www.forward.com.au
 *  All rights reserved.
 *  
 *  Redistribution of the source of this package, with or without
 *  modification, is NOT permitted.
 *  
 *  Redistribution of the compiled java .class files of this package, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions in compiled java .class form must reproduce the above
 *     copyright notice, this list of conditions and the following disclaimer
 *     in the documentation and/or other materials provided with the distribution.
 *  2. If the software is modified, the compiled java .class distribution must
 *     include, in the documentation and/or other materials provided with the 
 *     distribution, a statement that the software has been modified and
 *     identify the person who was responsable for the modifications.  
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
package au.com.forward.threads;

/**
 *  An exception of this class is thrown by the {@link ThreadReturn#join ThreadReturn.join()} methods when the 
 *  <code>join()</code> times out before the thread being joined dies.&nbsp;  It extends <code>InterruptedException</code> which
 * is thrown by <code>Thread.join()</code> so you do not need to change your <code>throws</code> specifications
 * when replacing <code>thread.join()</code> with <code>ThreadReturn.join()</code>.&nbsp;
 * See {@link ThreadReturn} for details.
 *  <p>
 *  The name of the thread being joined is set as this exception's message.&nbsp;
 *  You can use {@link StackTrace#toString StackTrace.toString(Throwable e)} to get this exception and its cause
 *  as a <code>String</code>
 * <p>
 *  There is no public constructor for this class as it is only used by 
 *  {@link ThreadReturn}. 
 *
 * @author     matthew ford 
 * @created    7 April 2002 
 */
public class TimedOutException extends InterruptedException {

	/**
	 *  Constructor for the ThreadException object 
	 *
	 * @param  message  the detailed message 
	 */
	TimedOutException(String message) {
		super(message);
	}
}